/*3:*/
#line 15 "pptran.w"

#include "psimpoll.h" 
#include "ppglobal.h" 

/*:3*//*4:*/
#line 20 "pptran.w"

/*7:*/
#line 152 "pptran.w"

static real geomean_s(real**logdat,int j,int nntax);
/*:7*//*9:*/
#line 181 "pptran.w"

static void tranlog_s(real**datain,real**dataout,int row,int col);
/*:9*//*11:*/
#line 255 "pptran.w"

static void trannorm_s(real**datain,real**dataout,int row,int col);
/*:11*//*13:*/
#line 286 "pptran.w"

static void transqrt_s(real**datain,real**dataout,int row,int col);
/*:13*//*15:*/
#line 312 "pptran.w"

static void transtan_s(real**datain,real**dataout,int row,int col);
/*:15*/
#line 21 "pptran.w"


/*:4*//*6:*/
#line 29 "pptran.w"

void datatran
(
int tran,
int nnsam,
int nntax,
real**smdat,
real*rsum,
real**trandat,
file_t*stat
)

{
static const char*trantext[]= 
{
"No transformation",
"Square-root transformation",
"Standardized to zero mean and unit variance",
"Normalized to unit length",
"Logarithm transformation",
"Centred log ratio transformation"
};

int n= 0;

loop_t j= 0;
loop_t kk= 0;

void(*tranfunc[])(real**,real**,int,int)= 
{NULL,transqrt_s,transtan_s,trannorm_s,tranlog_s};


real**tempdat= NULL;

if(tran==4&&(gl.code.dat=='%'||gl.recalc))n= 5;
else n= tran;

dprint(&(*stat),"%s%s\n",trantext[n],rsum==NULL?"":
": using modelled data");





if(rsum==NULL&&tran==0)
{
for(kk= 0;kk<nntax;kk++)
{
cpyrealarray(trandat[kk],smdat[kk],nnsam);
}

}

else if(rsum==NULL&&tran!=0)
{
tranfunc[tran](smdat,trandat,nntax,nnsam);
}

else if(rsum!=NULL&&tran==0)
{
for(j= 0;j<nnsam;j++)
{
real bnlsum= 0;
rtn= printf("%d ",j);
assert(rtn>0);
for(kk= 0;kk<nntax;kk++)
{
trandat[kk][j]= bnldev(smdat[kk][j],rsum[j]);
bnlsum+= trandat[kk][j];
}

for(kk= 0;kk<nntax;kk++)
{
if(bnlsum>FLT_MIN)trandat[kk][j]/= bnlsum;
else trandat[kk][j]= 0.0;
}

}

rtn= putchar('\n');
assert(rtn>0);

}

else if(rsum!=NULL&&tran!=0)
{
tempdat= amatreal(nntax,nnsam);
for(j= 0;j<nnsam;j++)
{
real bnlsum= 0;
rtn= printf("%d ",j);
assert(rtn>0);
for(kk= 0;kk<nntax;kk++)
{
tempdat[kk][j]= bnldev(smdat[kk][j],rsum[j]);
bnlsum+= tempdat[kk][j];
}

for(kk= 0;kk<nntax;kk++)
{
if(bnlsum>FLT_MIN)tempdat[kk][j]/= bnlsum;
else tempdat[kk][j]= 0.0;
}

}

rtn= putchar('\n');
assert(rtn>0);

tranfunc[tran](tempdat,trandat,nntax,nnsam);

fmatreal(tempdat,nntax);

}

rtn= fputs("Data transformation completed\n",gl.logf.fp);
assert(rtn!=EOF);

return;
}

/*:6*//*8:*/
#line 155 "pptran.w"

static real geomean_s
(
real**logdat,
int j,
int nntax
)

{
real geomean= 0;
loop_t k= 0;

for(k= 0;k<nntax;k++)geomean+= (loge(logdat[k][j]));
geomean= expe(geomean/(real)nntax);

return(geomean);
}

/*:8*//*10:*/
#line 184 "pptran.w"

static void tranlog_s
(
real**datain,
real**dataout,
int row,
int col
)


{
real cd= 0;
real geomean= 0;
real tol= 1e-3;

int c= 0;

loop_t k= 0;

for(col--;col>=0;col--)
{
if(gl.code.dat!='%'&&gl.recalc==OFF)
{
for(k= 0;k<row;k++)
{
dataout[k][col]= loge(datain[k][col]+1);
}

}

else
{


for(c= 0,k= 0;k<row;k++)
{
if(datain[k][col]<1e-4)c++;
}



cd= ((real)c+1.0)/((real)SQUARE(row));
for(k= 0;k<row;k++)
{
if(datain[k][col]<1e-4)
{
dataout[k][col]= tol*cd*((real)row-(real)c);
}
else dataout[k][col]= datain[k][col]-tol*cd;

}


geomean= geomean_s(dataout,col,row);


for(k= 0;k<row;k++)
{
dataout[k][col]= loge(dataout[k][col]/geomean);
}

}

}

return;
}

/*:10*//*12:*/
#line 258 "pptran.w"

static void trannorm_s
(
real**datain,
real**dataout,
int row,
int col
)

{
real sx= 0.0;
real sx2= 0.0;

loop_t i= 0;

for(col--;col>=0;col--)
{
sx2= 0.0;
for(i= 0;i<row;i++)sx2+= (SQUARE(datain[i][col]));
sx= sqrte(sx2);
for(i= 0;i<row;i++)dataout[i][col]= datain[i][col]/sx;
}

return;
}

/*:12*//*14:*/
#line 289 "pptran.w"

static void transqrt_s
(
real**datain,
real**dataout,
int row,
int col
)

{
loop_t i= 0;

for(col--;col>=0;col--)
{
for(i= 0;i<row;i++)dataout[i][col]= sqrte(datain[i][col]);
}

return;
}

/*:14*//*16:*/
#line 315 "pptran.w"

static void transtan_s
(
real**datain,
real**dataout,
int row,
int col
)

{
real sd= 0.0;
real sx= 0.0;
real sx2= 0.0;
real xbar= 0.0;
real xij= 0.0;

loop_t j= 0;

for(row--;row>=0;row--)
{
sx= sx2= 0.0;
for(j= 0;j<col;j++)
{
xij= datain[row][j];
sx+= xij;
sx2+= (SQUARE(xij));
}
if(sx>0.0)
{
xbar= sx/col;
sd= sqrte((sx2-SQUARE(sx)/(real)col)/((real)col-1.0));
for(j= 0;j<col;j++)dataout[row][j]= (datain[row][j]-xbar)/sd;
}

}

return;
}
/*:16*/
